property script_author : "Jonathan Nathan, JNSoftware LLC"
property script_name : "Sync Entourage & AB Groups"
property script_version : "1.2"
property script_release_date : "January 16, 2010"
property script_copyright : "Copyright @ 2010 JNSoftware LLC. All Rights Reserved."

(*
This script is free software; you can
redistribute it and/or modify it under
the terms of the GNU General
Public License as published by the
Free Software Foundation; either
version 2 of the License, or (at
your option) any later version.

This script is distributed in the hope 
that it will be useful, but WITHOUT 
ANY WARRANTY; without even the 
implied warranty of MERCHANTABILITY 
or FITNESS FOR A PARTICULAR 
PURPOSE. See the GNU General 
Public License for more details.

You should have received a copy of 
the GNU General Public License 
along with this script; if not, 
write to: 

		Free Software Foundation, Inc.
		59 Temple Place, Suite 330
		Boston, MA 02111-1307 USA
		
*)

property remove_from_existing_groups : false --set to true to delete group assignments in AB made independently of Entourage
property contacts_not_found : {}

set contacts_not_found to {}

activate
set remove_from_existing_groups to (button returned of (display dialog "Remove synced contacts from existing groups in the Address Book.app before syncing with Microsoft Entourage categories?" buttons {"Cancel", "No", "Yes"} default button 3 with icon 1) = "Yes")
activate
set remove_empty_groups to (button returned of (display dialog "Remove empty groups in the Address Book.app?" buttons {"Cancel", "No", "Yes"} default button 3 with icon 1) = "Yes")

set start_time to (current date)
tell application "Microsoft Entourage"
	set my_contact to me contact
	repeat with this_contact in (get contacts)
		tell this_contact's contents
			set is_me to (it = my_contact)
			set {contact_name, contact_company, contact_categories} to its {name, company, category}
			if contact_name = "" then set contact_name to contact_company
			set category_count to (count contact_categories)
			repeat with i from 1 to category_count
				set item i of contact_categories to (get name of item i of contact_categories)
			end repeat
			my update_contact(contact_name, contact_categories, is_me)
		end tell
	end repeat
end tell
if (remove_empty_groups) then
	tell application "Address Book"
		try
			repeat with this_group in (get groups)
				if (get people of this_group) = {} then delete this_group
			end repeat
		end try
		save
	end tell
end if
set cd to current date
set end_time to cd - start_time
beep
activate
if (contacts_not_found = {}) then
	set the_message to "Finished Syncing Entourage & AB Groups" & return & return & cd & return & return & "Total  time: " & end_time & " seconds"
	set the_buttons to {"OK"}
	set the_icon to 1
	set the_timeout to 10
else
	set contact_count to (count contacts_not_found)
	if (contact_count = 1) then
		set contact_count_insert to "was 1 contact"
	else
		set contact_count_insert to "were " & contact_count & " contacts"
	end if
	set the_message to "Finished Syncing Entourage & AB Groups" & return & return & cd & return & return & "Total  time: " & end_time & " seconds" & return & return & "There " & contact_count_insert & " found in Entourage but not in Address Book (the search uses the exact name or company name of the contact to find a match and this may differ slightly from Entourage to Address Book)." & return & return & "Do you want to save the list of contacts not found?"
	set the_buttons to {"No Thanks", "Save List of Missing Contacts"}
	set the_icon to 2
	set the_timeout to 60
end if
set the_button to button returned of (display dialog the_message buttons the_buttons default button (count the_buttons) with icon the_icon giving up after the_timeout)
if (the_button = "Save List of Missing Contacts") then
	try
		set contacts_not_found to my sort(contacts_not_found, false)
		set beginning of contacts_not_found to "Contacts found in Microsoft Entourage but not Address Book.app:"
		activate
		set the_file to POSIX path of (choose file name with prompt "Save the Missing Contact List file:" default name "Missing Contacts.txt")
		set ASCII_10 to ASCII character 10
		tell (a reference to my text item delimiters)
			set {o, contents} to {contents, ASCII_10}
			set {contacts_not_found, contents} to {contacts_not_found as Unicode text, o}
		end tell
		(do shell script "echo " & quoted form of contacts_not_found & " > " & (quoted form of the_file))
	end try
end if
return the_message

on update_contact(contact_name, contact_categories, is_me)
	try
		tell application "Address Book"
			try
				set the_contact to item 1 of (get people whose name = contact_name)
			on error
				if ((count contact_categories)  1) then
					set end of contacts_not_found to (contact_name & " (" & (my list_to_string(my sort(contact_categories, false), ", ")) & ")")
				else
					set end of contacts_not_found to contact_name
				end if
				error "Microsoft Entourage contact \"" & contact_name & "\" not found in Address Book.app."
			end try
			if (remove_from_existing_groups) then
				repeat with this_group in (get groups whose people of it contains the_contact)
					try
						remove the_contact from this_group
					end try
				end repeat
			end if
			repeat with this_category in contact_categories
				set this_category to this_category's contents
				try
					set this_group to item 1 of (get every group whose name = this_category)
				on error
					set this_group to make new group with properties {name:this_category}
				end try
				add the_contact to this_group
			end repeat
			if is_me then set my card to the_contact
			save
		end tell
	on error e number n
		log ("" & (current date) & " Error: " & e)
	end try
end update_contact

on sort(l, remove_duplicates)
	set ASCII_10 to ASCII character 10
	tell (a reference to my text item delimiters)
		set {o, contents} to {contents, ASCII_10}
		set {l, contents} to {l as Unicode text, o}
	end tell
	set the_parameters to "-f"
	if remove_duplicates then set the_parameters to the_parameters & "u"
	return (do shell script "echo " & quoted form of l & " | sort " & the_parameters)'s paragraphs
end sort

on list_to_string(l, d)
	tell (a reference to my text item delimiters)
		set {o, contents} to {contents, d}
		set {l, contents} to {l as Unicode text, o}
	end tell
	return l as Unicode text
end list_to_string
